// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.


#import "OSKDataSampleDestination.h"

#import <Foundation/NSData.h>
#import <Foundation/NSString.h>
#import <Foundation/NSException.h>
#import <OmniBase/OmniBase.h>
#import <OmniBase/system.h>

#import "SoundKit.h"
#import "OSKSampleSource.h"

RCS_ID("$Id: OSKDataSampleDestination.m,v 1.7.2.2 2000/09/12 19:05:41 bungi Exp $")

@implementation OSKDataSampleDestination

- init;
{
    [super init];

    sampleSource = nil;
    data = nil;

    return self;
}

- (void) dealloc;
{
    [sampleSource release];
    [data release];
}

- (void) setSampleSource: (OSKSampleSource *) source;
{
    if (sampleSource != source) {
        [sampleSource release];
        sampleSource = [source retain];

        [data release];
        data = nil;
    }
}

- (OSKSampleSource *) sampleSource;
{
    return sampleSource;
}

- (NSData *) run;
{
    OSKSampleIndex sampleIndex, frameCount, framesPerBuffer;
    OSKSampleIndex length;
    NSTimeInterval beforeInterval;
    
    if (!sampleSource) {
        data = [[NSData alloc] initWithLength: 0];
        return data;
    }

    frameCount = [sampleSource frameCount];
    framesPerBuffer = [sampleSource framesPerBuffer];

    length = frameCount * [sampleSource bytesPerFrame];
    data = [[NSMutableData alloc] initWithCapacity: length];

    // We can wait forever
    beforeInterval = [[NSDate distantFuture] timeIntervalSinceReferenceDate];
    
    for (sampleIndex = 0; sampleIndex < frameCount; sampleIndex += framesPerBuffer)
        [data appendData: [sampleSource sampleBufferStartingAtSampleIndex: sampleIndex
                                                       beforeTimeInterval: beforeInterval]];

#warning FIXME: Need to find out why this fails
    //OBPOSTCONDITION(length == [data length]);

    return data;
}

- (void) saveContentsAsSNDFileAtPath: (NSString *) path;
{
    SNDSoundStruct soundStruct;
    int fd;

    OBPRECONDITION(path);

    if ((fd = open([path cString], O_CREAT | O_WRONLY, 0666)) < 0)
        [NSException raise: @"OSKCannotOpenFile"
                    format: @"Couldn't open %@ to save into", path];

    soundStruct.magic = SND_MAGIC;
    soundStruct.dataLocation = sizeof(soundStruct);
    soundStruct.dataSize = 0; /* Not required */
    soundStruct.samplingRate = [sampleSource samplingRate];
    soundStruct.channelCount = [sampleSource samplesPerFrame];
    switch([sampleSource bytesPerSample]) {
        case 1:
            soundStruct.dataFormat = SND_FORMAT_LINEAR_8;
            break;
        case 2:
            soundStruct.dataFormat = SND_FORMAT_LINEAR_16;
            break;
        case 3:
            soundStruct.dataFormat = SND_FORMAT_LINEAR_24;
            break;
        default:
            soundStruct.dataFormat = SND_FORMAT_UNSPECIFIED;
            break;
    }

    if (write(fd, &soundStruct, sizeof(soundStruct)) != sizeof(soundStruct))
        [NSException raise: @"OSKWriteFailed" format: @"Failed to write sound header"];
    if (write(fd, [data bytes], [data length]) != [data length])
        [NSException raise: @"OSKWriteFailed" format: @"Failed to write sound data"];
}

@end
