// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OSKDataSampleSource.h"

#import <Foundation/NSData.h>
#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <OmniBase/OmniBase.h>

RCS_ID("$Id: OSKDataSampleSource.m,v 1.8.2.1 2000/09/12 18:41:25 bungi Exp $")

@implementation OSKDataSampleSource

- initWithSampleData: (NSData *) newSampleData
        samplingRate: (OSKSamplingRate) newSamplingRate
            channels: (int)newSamplesPerFrame
      bytesPerSample: (OSKSampleSize) newBytesPerSample
     framesPerBuffer: (OSKSampleIndex) newFramesPerBuffer
{
    OSKSampleIndex newFrameCount;

    OBPRECONDITION(newSampleData);
    OBPRECONDITION(newFramesPerBuffer); /* this is redundant, but we don't want the below to crash */
    OBPRECONDITION(newBytesPerSample * newSamplesPerFrame); /* this is redundant, but we don't want the below to crash */
    OBPRECONDITION([newSampleData length]);
    OBPRECONDITION([newSampleData length] % (newBytesPerSample * newSamplesPerFrame) == 0);

    newFrameCount = [newSampleData length] / (newBytesPerSample * newSamplesPerFrame);

    if (![self initWithFrameCount: newFrameCount
                     samplingRate: newSamplingRate
                         channels: newSamplesPerFrame
                   bytesPerSample: newBytesPerSample
                  framesPerBuffer: newFramesPerBuffer])
        return nil;

    sampleData = [newSampleData copy];
    sampleDataLength = [sampleData length];

    return self;
}

- (void) dealloc;
{
   [sampleData release];
   [super dealloc];
}

- (NSData *) sampleData;
{
    return sampleData;
}

#warning FIXME: We should really have our own subclass of NSData that we keep a free list of
#warning TODO: Might want to change the API to get rid of the autorelease

- (NSData *) sampleBufferStartingAtSampleIndex: (OSKSampleIndex) sampleIndex
                            beforeTimeInterval: (NSTimeInterval) beforeInterval;
{
    NSRange sampleRange;

    sampleRange.location = sampleIndex * _samplesPerFrame * _bytesPerSample;
    sampleRange.length = _framesPerBuffer * _samplesPerFrame * _bytesPerSample;

    if (sampleRange.location >= sampleDataLength)
	[NSException raise: @"OSKBadIndex"
             format: @"Asked for sample data at index %d of %@, but it only has %d samples!",
            sampleIndex, self, _frameCount];

    /* Possibly make the last buffer smaller than the others */
    if (sampleRange.location + sampleRange.length >= sampleDataLength)
        sampleRange.length = sampleDataLength - sampleRange.location;

    return [sampleData subdataWithRange: sampleRange];
}

@end
