// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OSKSampleBuffer.h"

#import <OmniFoundation/OFFastMutableData.h>
#import <OmniBase/OmniBase.h>

RCS_ID("$Id: OSKSampleBuffer.m,v 1.7.2.1 2000/09/12 18:41:26 bungi Exp $")

@implementation OSKSampleBuffer

+ (id) silentSampleBufferOfLength: (OSKSampleIndex) sampleCount
                     samplingRate: (OSKSamplingRate) samplingRate
                 numberOfChannels: (unsigned char) numberOfChannels
                   bytesPerSample: (unsigned char) bytesPerSample
                     startingTick: (OSKTick) startingTick;
{
    unsigned int        dataLength;
    OSKSampleBuffer    *zeroSampleBuffer;
    OFFastMutableData  *zeroSampleData;
    
    dataLength = sampleCount * numberOfChannels * bytesPerSample;

    // We have to zero the data ourselves since OFFastMutableData doesn't do it.
    zeroSampleData = [OFFastMutableData newFastMutableDataWithLength: dataLength];
    memset([zeroSampleData mutableBytes], 0, dataLength);
    
    zeroSampleBuffer = [[[OSKSampleBuffer alloc] initWithSampleData: zeroSampleData
                                                       samplingRate: OSK44100SamplingRate
                                                   numberOfChannels: 2
                                                     bytesPerSample: sizeof(OSKSample)
                                                       startingTick: startingTick] autorelease];
    [zeroSampleData release];

    return zeroSampleBuffer;
}

/*" Creates a new instance.  The given sampleData is NOT copied as sample data can be large and copying it expensive.  Rather, it is simply retained.  You should design your sample data generation algorithms to not overwrite buffers (at least until there are no more OSKSampleBuffers referencing them.
"*/
- initWithSampleData: (OFFastMutableData *) sampleData
        samplingRate: (OSKSamplingRate) samplingRate
    numberOfChannels: (unsigned char) numberOfChannels
      bytesPerSample: (unsigned char) bytesPerSample
        startingTick: (OSKTick) startingTick;
{
    _sampleData = [sampleData retain];
    _samplingRate = samplingRate;
    _numberOfChannels = numberOfChannels;
    _bytesPerSample = bytesPerSample;
    _startingTick = startingTick;

#warning Should check that there are no bytes left over
    _numberOfSamples = [_sampleData length] / (_numberOfChannels * _bytesPerSample);
    
    return self;
}

- (void) dealloc;
{
    [_sampleData release];
    [super dealloc];
}

- (OFFastMutableData *) sampleData;
{
    return _sampleData;
}

- (OSKSamplingRate) samplingRate;
{
    return _samplingRate;
}

- (unsigned char) numberOfChannels;
{
    return _numberOfChannels;
}

- (unsigned char) bytesPerSample;
{
    return _bytesPerSample;
}

- (OSKTick) startingTick;
{
    return _startingTick;
}

- (OSKTick) endingTick;
{
    return _startingTick + OSKSamplesToTicks(_numberOfSamples, _samplingRate);
}

- (OSKSampleIndex) numberOfSamples;
{
    return _numberOfSamples;
}

- (NSTimeInterval) durationSeconds;
{
    return OSKSamplesToSeconds(_numberOfSamples, _samplingRate);
}

- (OSKTick) durationTicks;
{
    return OSKSamplesToTicks(_numberOfSamples, _samplingRate);
}

@end
