// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
/* $Id: OSKSampleSource.h,v 1.12.2.1 2000/09/12 18:41:27 bungi Exp $ */

#import <Foundation/NSDate.h>
#import <OmniFoundation/OFObject.h>

@class NSData;

#import "OSKTypes.h"

@interface OSKSampleSource : OFObject
{
    OSKSampleIndex      _frameCount;        // Size if known, or INVALID
    OSKSampleIndex      _framesPerBuffer;   // Size of returned data
    OSKSamplingRate     _samplingRate;      // Sampling rate (e.g. 44100 Hz)
    unsigned short int  _samplesPerFrame;   // AKA channels
    OSKSampleSize       _bytesPerSample;    // OSKSampleSource only handles linear formats
}

- initWithSampleCount: (OSKSampleIndex) newSampleCount
         samplingRate: (OSKSamplingRate) newSamplingRate
       bytesPerSample: (OSKSampleSize) newBytesPerSample
     samplesPerBuffer: (OSKSampleIndex) newSamplesPerBuffer;

- initWithFrameCount: (OSKSampleIndex) newFrameCount
        samplingRate: (OSKSamplingRate) newSamplingRate
            channels: (int) newSamplesPerFrame
      bytesPerSample: (OSKSampleSize) newBytesPerSample
     framesPerBuffer: (OSKSampleIndex) newFramesPerBuffer;


- (OSKSampleIndex) sampleCount;
- (OSKSampleIndex) frameCount;


- (OSKSamplingRate) samplingRate;

- (OSKSampleSize) bytesPerFrame;

- (int)samplesPerFrame;

- (OSKSampleSize) bytesPerSample;

- (OSKSampleIndex) framesPerBuffer;

- (NSTimeInterval) sampleSourceDuration;

//
// Methods that must be subclassed
//

- (NSData *) sampleBufferStartingAtSampleIndex: (OSKSampleIndex) sampleIndex
                            beforeTimeInterval: (NSTimeInterval) beforeInterval;

@end
