// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OSKSampleSourceStream.h"

#import "OSKSampleSource.h"
#import "OSKSampleBuffer.h"

#import <OmniBase/OmniBase.h>

RCS_ID("$Id: OSKSampleSourceStream.m,v 1.2.2.1 2000/09/12 18:41:27 bungi Exp $")

@implementation OSKSampleSourceStream

- initWithSampleSource: (OSKSampleSource *) sampleSource;
{
    if (!(self = [super init]))
        return nil;
    
    _sampleSource = [sampleSource retain];
    _lastSampleIndex = [_sampleSource sampleCount];
    _nextSampleIndex = 0;
    _nextTick = 0;
    _samplingRate = [_sampleSource samplingRate];
    _numberOfChannels = [_sampleSource samplesPerFrame];
    _bytesPerSample = [_sampleSource bytesPerSample];

    return self;
}

- (void) dealloc;
{
    [_sampleSource release];
    [super dealloc];
}

- (OSKSampleSource *) sampleSource;
{
    return _sampleSource;
}

- (OSKSampleIndex) nextSampleIndex;
{
    return _nextSampleIndex;
}

- (void) setNextSampleIndex: (OSKSampleIndex) nextSampleIndex;
{
    if (_nextSampleIndex >= [_sampleSource sampleCount])
        [NSException raise: NSInvalidArgumentException
                    format: @"Sample index out of range."];
    
    _nextSampleIndex = nextSampleIndex;
}

//
// Subclass methods
//

- (OSKSampleBuffer *) nextSampleBufferByInterval: (NSTimeInterval) deadlineInterval
                             minimumStartingTick: (OSKTick) minimumStartingTick;
{
    NSData          *sampleData;
    OSKSampleBuffer *sampleBuffer;
    OSKSampleIndex   samplesInData;
    OSKTick          startingTick;
    
    do {
        if (_nextSampleIndex >= _lastSampleIndex) {
            sampleBuffer = nil;
            break;
        }

        sampleData = [_sampleSource sampleBufferStartingAtSampleIndex: _nextSampleIndex
                                                   beforeTimeInterval: deadlineInterval];

        samplesInData = [sampleData length] / (_numberOfChannels * _bytesPerSample);

        startingTick = _nextTick;
        sampleBuffer = [[[OSKSampleBuffer alloc] initWithSampleData: sampleData
                                                       samplingRate: _samplingRate
                                                   numberOfChannels: _numberOfChannels
                                                     bytesPerSample: _bytesPerSample
                                                       startingTick: _nextTick] autorelease];

        _nextSampleIndex += samplesInData;
        _nextTick += OSKSamplesToTicks(samplesInData, _samplingRate);
    } while (startingTick < minimumStartingTick);
    
    return sampleBuffer;
}

- (BOOL) isAtEnd
{
    return (_nextSampleIndex >= _lastSampleIndex);
}

@end
