// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OSKSineWaveSampleSource.h"
#import <Foundation/Foundation.h>

#import <OmniBase/OmniBase.h>

RCS_ID("$Id: OSKSineWaveSampleSource.m,v 1.7.2.1 2000/09/12 18:41:28 bungi Exp $")

#define AMPLITUDE (3000.0)


@implementation OSKSineWaveSampleSource

- initWithFrequency: (double) aFrequency
          amplitude: (double) anAmplitude
              phase: (double) aPhase
    framesPerBuffer: (OSKSampleIndex) newSamplesPerBuffer;
{
    if (![super initWithFrameCount: OSK_INVALID_SAMPLE_INDEX
                      samplingRate: OSK44100SamplingRate
                          channels: 2
                    bytesPerSample: sizeof(OSKSample)
                   framesPerBuffer: newSamplesPerBuffer])
        return nil;

    frequency = aFrequency;
    amplitude = anAmplitude * 32768.0;
    phase     = aPhase;
    
    return self;
}

- (double) frequency;
{
    return frequency;
}

- (double) amplitude;
{
    return amplitude;
}

- (double) phase;
{
    return phase;
}

- (NSData *) sampleBufferStartingAtSampleIndex: (OSKSampleIndex) sampleIndex;
{
    NSMutableData  *sampleData;
    OSKSample      *samples, sampleValue;
    OSKSampleIndex  lastSampleIndex;
            
    sampleData = [NSMutableData dataWithLength: _framesPerBuffer * [self bytesPerFrame]];
    samples = [(NSMutableData *)sampleData mutableBytes];

    lastSampleIndex = sampleIndex + _framesPerBuffer;
    while (sampleIndex < lastSampleIndex) {
        double t;

        t = (double)sampleIndex / 44100.0;

        sampleValue = amplitude * sin(2.0 * M_PI * t * frequency + phase);

        //fprintf(stderr, "sampleValue[%d] = %d, t = %f\n", (unsigned int)sampleIndex, (int)sampleValue, t);

        *samples++ = sampleValue; // left
        *samples++ = sampleValue; // right

        sampleIndex++;
    }

    return sampleData;
}

@end
