// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
/* $Id: OSKSoundOutSampleDestination.h,v 1.16.2.1 2000/09/12 18:41:29 bungi Exp $ */

#import <OmniFoundation/OFObject.h>

#import "OSKSampleStream.h"
#import "OSKTypes.h"

@class OSKSampleStream;
@class OFSlotManager;
@class NSPort, NSLock;

typedef enum _OSKPlayState {
    OSKStoppedPlayState,
    OSKPausedPlayState,
    OSKPlayingPlayState
} OSKPlayState;

@interface OSKSoundOutSampleDestination : OFObject
{
    id                _delegate;
    unsigned int      _numberOfBuffers;
    NSTimeInterval    _baseTimeInterval;
    struct mutex     *_mutex;
    struct condition *_condition;
    
    NSPort           *_locked_devicePort;
    NSPort           *_locked_ownerPort;
    NSPort           *_locked_streamPort;
    OSKPlayState      _locked_playState;
    OSKSampleStream  *_locked_sampleStream;

    void             *_locked_recordQueue;
    unsigned int      _locked_nextFinishingQueueEntry;
    unsigned int      _locked_nextTag;
    OSKTick           _nextTick;
    OSKTick           _lastCompletedTick;

    NSLock           *_pauseLock;
    unsigned int      _locked_pauseCount;

    unsigned int      _insertSamplesAdjustmentCount;
    unsigned int      _removeSamplesAdjustmentCount;
    
    NSLock           *_doneLock;
}

+ (OSKSoundOutSampleDestination *) sharedSoundOutSampleDestination;
+ (void) setSoundLowWaterBuffers: (OSKSampleIndex) newLowWaterBuffers;
+ (void) setSoundHighWaterBuffers: (OSKSampleIndex) newHighWaterBuffers;

- (void) setDelegate: (id) delegate;
- (id) delegate;


- (void) setNumberOfBuffers: (unsigned int) bufferCount;
- (unsigned int) numberOfBuffers;

- (void) setSampleStream: (OSKSampleStream *) newSampleStream
        baseTimeInterval: (NSTimeInterval) baseTimeInterval;
- (OSKSampleStream *) sampleStream;
- (NSTimeInterval) baseTimeInterval;

- (NSTimeInterval) averageSynchronizationOffset;
- (NSTimeInterval) enqueuedSampleDuration;
- (unsigned int) insertSamplesAdjustmentCount;
- (unsigned int) removeSamplesAdjustmentCount;

/* Add other accessors later */
- (void) start;
- (void) stop;

- (BOOL) setLeftVolume: (int) left  rightVolume: (int) right;
- (BOOL) getLeftVolume: (int *) left rightVolume: (int *) right;

- (void) seekToSample: (OSKSampleIndex) sampleIndex;
- (OSKTick) nextTick;
- (OSKTick) lastCompletedTick;

- (void) pause;
- (void) resume;
- (unsigned int) pauseCount;

- (OSKPlayState) playState;

- (void) finish;



@end


@interface NSObject (OSKSoundOutSampleDestinationDelegate)

- (OSKSampleStream *) nextSampleStream;
/*.doc.
This is called when the OSKSoundOutSampleDestination is playing and has exhausted its current sample source.  If this returns non-nil, the returned sample source is made the current sample source and play continues with the samples contained therein.

The returned sample source, if non-nil, must have the same parameters (sampling rate, channel count, data format) as the previous one.
*/

/*" This is called whenever the play state of the sound-out changed.  This will always be called in the main thread. "*/
- (void) playStateChanged;

@end
