// Copyright 1997-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
/* $Id: OSKTypes.h,v 1.4.2.1 2000/09/12 18:41:29 bungi Exp $ */

#import <Foundation/Foundation.h>


typedef unsigned char      OSKSampleSize;
// TJW -- Actually, I think we should use signed short
typedef unsigned short     OSKSample;
typedef unsigned long long OSKSampleIndex;


#define OSK_INVALID_SAMPLE_INDEX (OSKSampleIndex)(-1)
#define OSK_MIN_SAMPLE           (0)
#define OSK_MAX_SAMPLE           (OSKSample)(-1)


// These are the only sampling rates that OSK supports right now.
typedef enum _OSKSamplingRate {
    OSK44100SamplingRate = 44100,
    OSK22050SamplingRate = 22050,
} OSKSamplingRate;



static inline OSKSamplingRate OSKValidateSamplingRate(unsigned int rate)
{
    switch (rate) {
        case OSK44100SamplingRate:
        case OSK22050SamplingRate:
            return (OSKSamplingRate) rate;
        default:
            [NSException raise: NSInvalidArgumentException
                        format: @"Illegal samplingRate of '%d'.", rate];
            return (OSKSamplingRate)0;
    }
}

// All times in OmniAmp are represented in 'ticks'.  A 'tick' value
// describes the numerator of a fraction that is the actual number
// of seconds.  This allows times to be added w/o any floating point
// round-off accumulation issues since the conversion to the floating
// point seconds value is the last step in computing the time.
typedef unsigned long long OSKTick;

// The denominator in this fraction should be the GCD of all the
// sampling rates.
#define OSK_TICKS_PER_SECOND (44100.0)
#define OSK_SECONDS_PER_TICK (1.0/OSK_TICKS_PER_SECOND)

#define OSK_INVALID_TICK ((OSKTick)~0)

static inline NSTimeInterval OSKTicksToSeconds(OSKTick tick)
{
    return tick * OSK_SECONDS_PER_TICK;
}

static inline OSKTick OSKSecondsToTicks(NSTimeInterval seconds)
{
    return (OSKTick)(seconds * OSK_TICKS_PER_SECOND);
}

static inline OSKTick OSKSamplesToTicks(OSKSampleIndex sampleCount,
                                        OSKSamplingRate samplingRate)
{
    if (sampleCount == OSK_INVALID_SAMPLE_INDEX)
        return OSK_INVALID_TICK;

    switch (samplingRate) {
        case OSK44100SamplingRate:
            return sampleCount;
        case OSK22050SamplingRate:
            return 2 * sampleCount;
        default:
            [NSException raise: NSInvalidArgumentException
                        format: @"Illegal samplingRate of '%d'.", samplingRate];
            return (OSKTick)0;
    }
}

static inline OSKSampleIndex OSKTicksToSamples(OSKTick ticks,
                                               OSKSamplingRate samplingRate)
{
    if (ticks == OSK_INVALID_TICK)
        return OSK_INVALID_SAMPLE_INDEX;

    switch (samplingRate) {
        case OSK44100SamplingRate:
            return ticks;
        case OSK22050SamplingRate:
            return ticks / 2;
        default:
            [NSException raise: NSInvalidArgumentException
                        format: @"Illegal samplingRate of '%d'.", samplingRate];
            return 0;
    }
}


static inline NSTimeInterval OSKSamplesToSeconds(OSKSampleIndex sampleCount,
                                                 OSKSamplingRate samplingRate)
{
    return OSKTicksToSeconds(OSKSamplesToTicks(sampleCount, samplingRate));
}
