// Copyright 2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
/* $Id: SoundKit.h,v 1.1.2.1 2000/09/12 19:05:43 bungi Exp $ */



// This file encompasses the small portion of the NEXTSTEP 3.3 SoundKit API that we use in OmniSoundKit.  It seems to still be valid up to MacOS X Server.

#import <mach/mach.h>

typedef struct {
    int magic;
    int dataLocation;
    int dataSize;
    int dataFormat;
    int samplingRate;
    int channelCount;
    char info[4];
} SNDSoundStruct;

#define SND_MAGIC ((int)0x2e736e64)
#define SND_FORMAT_UNSPECIFIED          (0)
#define SND_FORMAT_LINEAR_8             (2)
#define SND_FORMAT_LINEAR_16            (3)
#define SND_FORMAT_LINEAR_24            (4)


#define SND_ACCESS_OUT 1

typedef struct {
    int priority;
    int duration;
} SNDNegotiationData;

typedef enum {
    SND_ERR_NONE = 0
} SNDError;
char *SNDSoundError(int err);

typedef int (*SNDNegotiationFun)(void *arg, SNDNegotiationData *data);

int SNDAcquire(int access_code, int priority, int preempt, int timeout,
                SNDNegotiationFun negotiation_function, void *arg,
                port_t *device_port, port_t *owner_port);
int SNDRelease(int access_code, port_t device_port, port_t owner_port);


typedef void (*sndreply_tagged_t)(void *arg, int tag);
typedef void (*sndreply_recorded_data_t)(void *arg, int tag,
                                         void *data, int size);
typedef void (*sndreply_dsp_cond_true_t)(void *arg, u_int mask,
                                         u_int flags, u_int regs);
typedef void (*sndreply_dsp_msg_t)(void *arg, int *data, int size);

typedef struct snddriver_handlers {
    void *			arg;
    int				timeout;
    sndreply_tagged_t		started;
    sndreply_tagged_t		completed;
    sndreply_tagged_t		aborted;
    sndreply_tagged_t		paused;
    sndreply_tagged_t		resumed;
    sndreply_tagged_t		overflow;
    sndreply_recorded_data_t	recorded_data;
    sndreply_dsp_cond_true_t	condition_true;
    sndreply_dsp_msg_t		dsp_message;
    sndreply_dsp_msg_t		dsp_error;
 } snddriver_handlers_t;

kern_return_t snddriver_stream_setup (
        port_t		dev_port,
        port_t		owner_port,
        int		config,
        int		buf_size,
        int		sample_size,
        int		low_water,
        int		high_water,
        int		*protocol,
        port_t		*stream_port);

kern_return_t snddriver_set_sndout_bufcount (
        port_t		dev_port,
        port_t		owner_port,
        int		sobcount);

kern_return_t snddriver_stream_control (
        port_t		stream_port,
        int		tag,
        int		snd_control);

kern_return_t snddriver_get_volume (
        port_t		device_port,
        int		*left_chan,
        int		*right_chan);

kern_return_t snddriver_set_volume (
        port_t		device_port,
        int		left_chan,
        int		right_chan);

kern_return_t snddriver_stream_start_writing (
        port_t		stream_port,
        void		*data,
        int		data_size,
        int		tag,
        boolean_t	preempt,
        boolean_t	deallocate,
        boolean_t	started_msg,
        boolean_t	completed_msg,
        boolean_t	aborted_msg,
        boolean_t	paused_msg,
        boolean_t	resumed_msg,
        boolean_t	overflow_msg,
        port_t		reply_port);

kern_return_t snddriver_reply_handler(
        msg_header_t		*msg,
        snddriver_handlers_t	*handlers);

#define SNDDRIVER_STREAM_TO_SNDOUT_44	(3)
#define SNDDRIVER_ABORT_STREAM		(0x2)
#define SNDDRIVER_PAUSE_STREAM		(0x4)
#define SNDDRIVER_RESUME_STREAM		(0x8)

