
#import <Foundation/Foundation.h>

#import <OmniSoundKit/OmniSoundKit.h>
#import <OmniBase/OBPostLoader.h>

const char *pgm = NULL;

volatile void usage()
{
    fprintf(stderr,
            "usage: %s play sndfile1 ... sndfileN\n"
            "       %s copy sndfile1 sndfile2\n"
            "       %s sequence sndfile1 startTime1 ... sndfileN startTimeN outputSNDFile\n"
            "       %s sin frequency amplitude phase\n",
            pgm, pgm, pgm, pgm);
    exit(1);
}

void sequence(int argc, const char *argv[])
{
    OSKCompositeSampleSource *compositeSource;


    if ((argc == 1) || (argc % 2 != 1))
        usage();

    compositeSource = [[OSKCompositeSampleSource alloc] initSamplingRate: OSK44100SamplingRate
                                                          bytesPerSample: 2 * sizeof(OSKSample)
                                                        samplesPerBuffer: NSPageSize() / (2 * sizeof(OSKSample))];

    while (argc != 1) {
        NSString *path;
        OSKSampleSource *fileSource;
        unsigned int startSampleIndex;
        float startTime;

        path = [[NSString alloc] initWithCString: *argv];
        argc--; argv++;

        fileSource = [OSKSNDFileSampleSource sampleSourceWithSNDFileAtPath: path
                                                           framesPerBuffer: NSPageSize()
                                                        sequentialReadOnly: NO];
        [path release];

        startTime = atof(*argv);
        argc--; argv++;

        startSampleIndex = startTime * OSK44100SamplingRate;

        [compositeSource addSubsidiarySampleSource: fileSource startSampleIndex: startSampleIndex];
    }


    {
        NSString *outputSNDPath;
        OSKDataSampleDestination *dataDest;

        dataDest = [[OSKDataSampleDestination alloc] init];
        [dataDest setSampleSource: compositeSource];
        [dataDest run];

        outputSNDPath = [[NSString alloc] initWithCString: *argv];
        argc--; argv++;

        [dataDest saveContentsAsSNDFileAtPath: outputSNDPath];
        [outputSNDPath release];
    }

    [compositeSource release];
}

void play(int argc, const char *argv[])
{
    OSKSampleSource *sampleSource;
    OSKSampleSourceStream *sampleStream;
    OSKSoundOutSampleDestination *soundOut;

    soundOut = [OSKSoundOutSampleDestination sharedSoundOutSampleDestination];

    while (argc--) {
        NSString *path;

        path = [[NSString alloc] initWithCString: *argv];
        sampleSource = [OSKSNDFileSampleSource sampleSourceWithSNDFileAtPath: path
                                                             framesPerBuffer: NSPageSize()
                                                          sequentialReadOnly: NO];
        sampleStream = [[[OSKSampleSourceStream alloc] initWithSampleSource: sampleSource] autorelease];
            
        [path release];
        argv++;

        [soundOut setSampleStream: sampleStream
                 baseTimeInterval: [NSDate timeIntervalSinceReferenceDate] + 1.0];  // let one second queue up
        [soundOut start];
        [soundOut finish];
    }
}

void do_sin(int argc, const char *argv[])
{
    OSKSampleSource *sampleSource;
    OSKSoundOutSampleDestination *soundOut;
    OSKSampleSourceStream *sampleStream;
    double frequency, amplitude, phase;
    
    if (argc != 3)
        usage();
    
    soundOut = [OSKSoundOutSampleDestination sharedSoundOutSampleDestination];

    frequency = atof(argv[0]);
    amplitude = atof(argv[1]);
    phase     = atof(argv[2]);
    fprintf(stderr, "frequency = %f, amplitude = %f, phase = %f\n", frequency, amplitude, phase);

    sampleSource = [[OSKSineWaveSampleSource alloc] initWithFrequency: frequency
                                                            amplitude: amplitude
                                                                phase: phase];
    sampleStream = [[[OSKSampleSourceStream alloc] initWithSampleSource: sampleSource] autorelease];

    [soundOut setSampleStream: sampleStream
             baseTimeInterval: [NSDate timeIntervalSinceReferenceDate] + 1.0];  // let one second queue up
    [soundOut start];
    [soundOut finish];
}

void copy(int argc, const char *argv[])
{
    OSKSampleSource *sampleSource;
    OSKDataSampleDestination *dataDest;
    NSString *path;

    if (argc != 2)
        usage();

    path = [[NSString alloc] initWithCString: *argv];
    argc--; argv++;

    sampleSource = [OSKSNDFileSampleSource sampleSourceWithSNDFileAtPath: path
                                                         framesPerBuffer: NSPageSize()
                                                      sequentialReadOnly: YES];
    [path release];

    dataDest = [[OSKDataSampleDestination alloc] init];
    [dataDest setSampleSource: sampleSource];
    [dataDest run];

    path = [[NSString alloc] initWithCString: *argv];
    argc--; argv++;
    [dataDest saveContentsAsSNDFileAtPath: path];
    [path release];
}

int main (int argc, const char *argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    [OBPostLoader processClasses];

    pgm = argv[0];
    argc--; argv++;

    if (argc < 1)
        usage();

    if (!strcmp(*argv, "play")) {
        argc--; argv++;
        play(argc, argv);
    } else if (!strcmp(*argv, "copy")) {
        argc--; argv++;
        copy(argc, argv);
    } else if (!strcmp(*argv, "sequence")) {
        argc--; argv++;
        sequence(argc, argv);
    } else if (!strcmp(*argv, "sin")) {
        argc--; argv++;
        do_sin(argc, argv);
    } else
        usage();

    [pool release];
    exit(0);       // insure the process exit status is 0
    return 0;      // ...and make main fit the ANSI spec.
}
